/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
#ifndef _RESOURCE_H_
#define _RESOURCE_H_

#include "libadminutil/admutil.h"

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************************/

/* Resource contains the name of the
   property file w/ paht information
*/
typedef struct
{
	char *path;
	char *package;
    void *propset;
} Resource;

/* Set the search root for NLS property file
   return: 1 - OK
           0 - Failed
   */
PR_IMPLEMENT(int)
res_init_path(const char* path);


/* Initialization routine.  Checks for the existence of a resourcebundle
   with the default encoding -  

   path: relative directory path to the properties file (ex, "server/property")
         which is set by res_init_path
   package: name of the properties file. (ex, "myresource")

   return value:  NULL if any memory allocation fails
				  NULL if the property file cannot be located
				  Resource* when successful
*/
PR_IMPLEMENT(Resource*)
res_init_resource(const char* path, const char* package);

/*  This routine retrieves a string from a property file that was initialized 
	to source.  It tries to locate a property file according to the 
	acceptlanguage.  

    source: pointer to Resource returned from initialization
	key: nueric key value.  Note than enum is generated from dbt*.h files.
	     For example, if you have Resdef(my_key, 1, "this is a test") in dbt*.h
		 file for your source, your key will be my_key.  

	return value: NULL if the property file cannot be located
				  NULL if any memory allocation fails.
				  const char* containing the string if all goes well.
*/
PR_IMPLEMENT(const char*) 
res_getstring(Resource* source, char *key, char *accept_language);

/* frees Resource* from initialization routine.
*/
PR_IMPLEMENT(void)
res_destroy_resource(Resource* to_destroy);

#ifdef __cplusplus
}
#endif


/*******************************************************************************/
/* 
 * this table contains library name
 * (stored in the first string entry, with id=0),
 * and the id/string pairs which are used by library  
 */

typedef struct res_RESOURCE_TABLE
{
  int id;
  char *str;
} res_RESOURCE_TABLE;

/*******************************************************************************/

/* 
 * resource global contains resource table list which is used
 * to generate the database.
 * Also used for "in memory" version of XP_GetStringFromDatabase()
 */

typedef struct res_RESOURCE_GLOBAL
{
  res_RESOURCE_TABLE  *restable;
} res_RESOURCE_GLOBAL;

/*******************************************************************************/

/*
 * Define the ResDef macro to simplify the maintenance of strings which are to
 * be added to the library or application header file (dbtxxx.h). This enables
 * source code to refer to the strings by theit TokenNames, and allows the
 * strings to be stored in the database.
 *
 * Usage:   ResDef(TokenName,TokenValue,String)
 *
 * Example: ResDef(DBT_HelloWorld_, \
 *                 1,"Hello, World!")
 *          ResDef(DBT_TheCowJumpedOverTheMoon_, \
 *                 2,"The cow jumped over the moon.")
 *          ResDef(DBT_TheValueOfPiIsAbout31415926536_, \
 *                 3,"The value of PI is about 3.1415926536."
 *
 * RESOURCE_STR is used by makstrdb.c only.  It is not used by getstrdb.c or
 * in library or application source code.
 */
 
#ifdef  RESOURCE_STR
#define BEGIN_STR(argLibraryName) \
                          RESOURCE_TABLE argLibraryName[] = { 0, #argLibraryName,
#define ResDef(argToken,argID,argString) \
                          argID, argString,
#define END_STR(argLibraryName) \
                          0, 0 };
#else
#define BEGIN_STR(argLibraryName) \
                          enum {
#define ResDef(argToken,argID,argString) \
                          argToken = argID,
#define END_STR(argLibraryName) \
                          argLibraryName ## top };
#endif



#endif /* _RESOURCE_H_ */
