/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*
 * Description (certmgt.c)
 *
 *        This module contains routines used by administration CGI
 *        programs to access certificate databases.
 */


#include "libadminutil/admutil.h"

#include "key.h"
#include "cert.h"
#include "certdb.h"
#include "secder.h"
#include "pk11func.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 *        This function should not be used as of NSS 3.2. Open of CERTDB is 
 *  done with NSS initialize functions.
 */
PR_IMPLEMENT(int)
cmgOpenCertDB(char * alias, CERTCertDBHandle **dbhandle)
{
    servssl_error("cmgOpenCertDB should not be used as of NSS 3.2\n");    
    return -1;
}

/*
 *        This function should not be used as of NSS 3.2. Close of CERTDB is 
 *  done with NSS_Shutdown() call.
 */
PR_IMPLEMENT(void)
cmgCloseCertDB(CERTCertDBHandle * handle)
{
    servssl_error("cmgCloseCertDB should not be used as of NSS 3.2\n");    
}

/* Get a representative string for a given SECName */
PR_IMPLEMENT(char *)
cmgGetNameDesc(CERTName * name)
{
    char * s;

    s = CERT_GetCommonName(name);
    if (s == 0) {
        s = CERT_GetOrgUnitName(name);
        if (s == 0) {
            s = CERT_GetOrgName(name);
            if (s == 0) {
                CERT_GetLocalityName(name);
                if (s == 0) {
                    s = CERT_GetStateName(name);
                    if (s == 0) s = "";
                }
            }
        }
      }

    return PORT_Strdup(s);
}

PR_IMPLEMENT(int)
cmgShowCerts(CERTCertDBHandle * handle, char * caption)
{
    servssl_error("cmgShowCerts() is not ported to NSS 3.2 as not used (HTML)\n");
    return  -1;
}


/* 
 * cmgShowCertsBySlot
 * Output a list of server certificates belonging to an external slot.
 * (no HTML output)
 */

PR_IMPLEMENT(int)
cmgShowCertsBySlot(PK11SlotInfo *slot)
{
    servssl_error("cmgShowCertsBySlot is not ported to NSS 3.2 as not used\n");
    return -1;
}


/*
 * cmgShowInternalCerts
 * Output a list of server certificates belonging to an internal slot.
 * (Unlike cmgShowCerts, this has no HTML output)
 */

PR_IMPLEMENT(int)
cmgShowInternalCerts(CERTCertDBHandle *handle)
{
    servssl_error("cmgShowInternalCerts is not ported to NSS 3.2 as not used\n");
    return -1;
}

static SECStatus cmgShowCrlRow(CERTSignedCrl *crl)
{
    char * datestr;
    char * nn;
/*    SECCertTimeValidity validity; */

    nn = cmgGetNameDesc(&crl->crl.name);
    if (!nn) {
        return SECFailure;
    }

    datestr = DER_UTCTimeToAscii(&crl->crl.nextUpdate);
/*    validity = SEC_CheckCrlTimes(&crl->crl, time(NULL)); */

    printf("%s;%s\n", nn, datestr);

    return SECSuccess;
}


PR_IMPLEMENT(int)
cmgShowCrls(CERTCertDBHandle *handle, int list_type)
{

  CERTCrlHeadNode *CrlListHead;
  CERTCrlNode *node;
  SECStatus rv;

  rv = SEC_LookupCrls(handle, &CrlListHead, list_type);
  if(rv != SECSuccess) return rv;

  for(node = CrlListHead->first; node != NULL; node = node->next) {
      cmgShowCrlRow(node->crl);
  }

  return SECSuccess;
}


#define BREAK "<br>"
#define BREAKLEN 4
#define COMMA ", "
#define COMMALEN 2

/********
  XXX Out of date, use at your own risk!
  We now use CERT_FormatName instead. -Adam
*********/

PR_IMPLEMENT(char *)
cmgFormatName(CERTName *name)
{
    CERTRDN** rdns;
    CERTRDN *rdn;
    CERTAVA** avas;
    CERTAVA* ava;
    char *buf = 0;
    char *tmpbuf = 0;
    unsigned len = 0;
    int tag;
    SECItem *cn, *email, *org, *orgunit, *loc, *state, *country;
    PRBool first;
    
    cn = 0;
    email = 0;
    org = 0;
    orgunit = 0;
    loc = 0;
    state = 0;
    country = 0;
    
    /* Loop over name components and gather the interesting ones */
    rdns = name->rdns;
    while ((rdn = *rdns++) != 0) {
        avas = rdn->avas;
        while ((ava = *avas++) != 0) {
            tag = CERT_GetAVATag(ava);
            switch(tag) {
              case SEC_OID_AVA_COMMON_NAME:
                cn = &ava->value;
                len += cn->len;
                break;
              case SEC_OID_AVA_COUNTRY_NAME:
                country = &ava->value;
                len += country->len;
                break;
              case SEC_OID_AVA_LOCALITY:
                loc = &ava->value;
                len += loc->len;
                break;
              case SEC_OID_AVA_STATE_OR_PROVINCE:
                state = &ava->value;
                len += state->len;
                break;
              case SEC_OID_AVA_ORGANIZATION_NAME:
                org = &ava->value;
                len += org->len;
                break;
              case SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME:
                orgunit = &ava->value;
                len += orgunit->len;
                break;
              case SEC_OID_PKCS9_EMAIL_ADDRESS:
                email = &ava->value;
                len += email->len;
                break;
              default:
                break;
            }
        }
    }

    /* XXX - add some for formatting */
    len += 128;

    /* allocate buffer */
    buf = (char *)PORT_Alloc(len);
    if ( !buf ) {
        return(0);
    }

    tmpbuf = buf;
    
    if ( cn ) {
        PORT_Memcpy(tmpbuf, cn->data + 2, cn->len - 2);
        tmpbuf += ( cn->len - 2 );
        PORT_Memcpy(tmpbuf, BREAK, BREAKLEN);
        tmpbuf += BREAKLEN;
    }
    if ( email ) {
        PORT_Memcpy(tmpbuf, email->data + 2, email->len - 2);
        tmpbuf += ( email->len - 2 );
        PORT_Memcpy(tmpbuf, BREAK, BREAKLEN);
        tmpbuf += BREAKLEN;
    }
    if ( orgunit ) {
        PORT_Memcpy(tmpbuf, orgunit->data + 2, orgunit->len - 2);
        tmpbuf += ( orgunit->len - 2 );
        PORT_Memcpy(tmpbuf, BREAK, BREAKLEN);
        tmpbuf += BREAKLEN;
    }
    if ( org ) {
        PORT_Memcpy(tmpbuf, org->data + 2, org->len - 2);
        tmpbuf += ( org->len - 2 );
        PORT_Memcpy(tmpbuf, BREAK, BREAKLEN);
        tmpbuf += BREAKLEN;
    }
    first = PR_TRUE;
    if ( loc ) {
        PORT_Memcpy(tmpbuf, loc->data + 2, loc->len - 2);
        tmpbuf += ( loc->len - 2 );
        first = PR_FALSE;
    }
    if ( state ) {
        if ( !first ) {
            PORT_Memcpy(tmpbuf, COMMA, COMMALEN);
            tmpbuf += COMMALEN;
        }
        PORT_Memcpy(tmpbuf, state->data + 2, state->len - 2);
        tmpbuf += ( state->len - 2 );
        first = PR_FALSE;
    }
    if ( country ) {
        if ( !first ) {
            PORT_Memcpy(tmpbuf, COMMA, COMMALEN);
            tmpbuf += COMMALEN;
        }
        PORT_Memcpy(tmpbuf, country->data + 2, country->len - 2);
        tmpbuf += ( country->len - 2 );
        first = PR_FALSE;
    }
    if ( !first ) {
        PORT_Memcpy(tmpbuf, BREAK, BREAKLEN);
        tmpbuf += BREAKLEN;
    }

    *tmpbuf = 0;

    return(buf);
}

PR_IMPLEMENT(char *)
cmgHTMLCertInfo(CERTCertificate *cert)
{
    servssl_error("cmgHTMLCertInfo() is not ported to NSS 3.2 as not used (HTML)\n");
   return NULL;
}


PR_IMPLEMENT(char *)
cmgHTMLCrlInfo(CERTSignedCrl *crl)
{
    char *issuer;
    char *notBefore, *notAfter;

#define NCHARPERLINE 50
#define NLINES 20
    const int output_buflen = NCHARPERLINE * NLINES;
    char * outputBuffer;
    
    if (!crl) {
        return(0);
    }

    issuer = CERT_FormatName(&crl->crl.name);
    notBefore = (char *)DER_UTCTimeToAscii(&crl->crl.lastUpdate);
    notAfter = (char *)DER_UTCTimeToAscii(&crl->crl.nextUpdate);

    /* reformat dates */
    notBefore[10] = ',';
    notAfter[10] = ',';
    if ( notBefore[8] == '0' ) {
        notBefore[8] = ' ';
    }
    if ( notAfter[8] == '0' ) {
        notAfter[8] = ' ';
    }

    outputBuffer = PR_smprintf("ISSUER=%s\nNOTBEFORE=%s\nNOTAFTER=%s\n",
                               issuer, notBefore, notAfter);
    
    if ( issuer ) {
        PORT_Free(issuer);
    }
    
    if ( notBefore ) {
        PORT_Free(notBefore);
    }
    
    if ( notAfter ) {
        PORT_Free(notAfter);
    }
    
    return(outputBuffer);
}


static SECStatus cmgFindCrl(CERTSignedCrl *crl, char *name)
{
    char * nn;

    nn = cmgGetNameDesc(&crl->crl.name);
    if (!nn) {
        return SECFailure;
    }

    if (strcmp(nn,name) == 0) {
      return SECSuccess; /* We found what we were looking for, 
                          * break from our loop */
    }
    return SECFailure;
}


/* 
 * cmgFindCrlByName(CERTCertDBHandle *, char *, int)
 *
 * Search both CRL's and CKL's for the list with name 'name'.
 */

PR_IMPLEMENT(CERTSignedCrl *)
cmgFindCrlByName(CERTCertDBHandle *handle, char *name, int list_type) 
{

  CERTCrlHeadNode *CrlListHead;
  CERTCrlNode *node;
  SECStatus rv;

  if((!name) || (!handle))
    return NULL;

  rv = SEC_LookupCrls(handle, &CrlListHead, list_type);
  if(rv != SECSuccess) return NULL;

  for(node = CrlListHead->first; node != NULL; node = node->next) {
      if(cmgFindCrl(node->crl, name) == SECSuccess)
        break;
  }

  if(node)
    /* OK, node contains the "dummy" crl - get the real one now */
    return(SEC_FindCrlByName(handle, &node->crl->crl.derName, list_type));
  else
    return NULL;
}


void
ByteRev(unsigned char *cpLENumber, int iSize)
{
  int iCursor = 0, iLimit;
  unsigned char cTemp;

  iLimit = iSize/2; /* Last byte to exchange */
  for (; iCursor<iLimit; iCursor++)
  {
    cTemp = *(cpLENumber+iCursor);
    *(cpLENumber+iCursor) = *(cpLENumber+iSize-iCursor-1);
    *(cpLENumber+iSize-iCursor-1) = cTemp;
  }

  return;
}
